/** 
 * @file  IO_ctrl.cpp
 * @brief Device I/O control Module
 *
 * @author Katayama
 * @date 2009-01-26
 * @version 1.00 : 2009/01/26 katayama
 * @version 2.01 : 2009/06/09 katayama (for TPIP2)
 * @version 3.01 : 2014/03/25 tanimoto (for TPIP3)
 *
 * Copyright (C) 2009 TPIP User Community All rights reserved.
 * ̃t@C̒쌠́ATPIP[U[R~jeB̋Kɏ]
 * gp܂B
 */

#include <stdio.h>
#include "system.h"
#include "IO_ctrl.h"
#include "IO_symbl.h"
#include "sub_module.h"
#include "com_sub.h"

//  =========================================
//
//		 ʊ֐@o[
//
//  =========================================

/** IOctrl ֐
 * @fn     void IO_ctrl::init(int dev_id, DEF_DEV_t* dev_nm)
 * @param  dev_id : Entry device ID
 * @param  dev_nm : foCX|C^
 *
 */
void IO_ctrl::init(int dev_id, DEF_DEV_t* dev_nm)
{

	if (dev_id >= 0)
	{
		memcpy( &dev_if, dev_nm, sizeof(DEF_DEV_t));
		mask = 0;
		if ((dev_if.dv_type == BUTT_)||(dev_if.dv_type == POV_)||(dev_if.dv_type == DO_)) 
				mask = (0x01 << dev_if.ch_no);	// set Button mask bit
	}
}

/** foCXo^|WVo
 * @fn     int IO_ctrl::get_pos(void)
 * @retval ==-1 : error
 * @retval >= 0 : o^|WV
 *
 */
int IO_ctrl::get_pos(void)
{
	int pos = -1;

	switch (dev_if.dv_type)
	{
	case AXIS_:
		pos = AXIS_TOP_POS;
		break;
	case BUTT_:
		pos = BUTT_TOP_POS;
		break;
	case POV_:
		pos = POV_TOP_POS;
		break;
	case PWM_:
		pos = PWM_TOP_POS;
		break;
	case PWM2_:
		pos = PWM2_TOP_POS;
		break;
	case DCMT_:
		pos = DCMT_TOP_POS;
		break;
	case DO_:
		pos = DO_TOP_POS;
		break;
	case VSEL_:
		pos = VSEL_TOP_POS;
		break;
	case VGA_:
		pos = VGA_TOP_POS;
		break;
	case MODE_:
		pos = MODE_TOP_POS;
		break;
	default:
		break;
	}

	if (pos > -1)  pos = pos + dev_if.ch_no;
	return (pos);
}

/** foCX őY ݒ
 * @fn     void IO_ctrl::set_dev_n(int n_)
 * @param n  : őY
 *
 */
void IO_ctrl::set_dev_n(int n_)
{
	dev_if.n = n_;	// őY Zbg
}

/** foCX sђl ݒ
 * @fn     void IO_ctrl::set_dzone(int v)
 * @param v  : sђli0 ` 100%j
 *
 */
void IO_ctrl::set_dzone(int v)
{

	dzone = Limit( v , 0, 100 );
}

/** foCX ől ݒ
 * @fn     void IO_ctrl::set_max(int v)
 * @param v  : ől
 *
 */
void IO_ctrl::set_max(int v)
{
	int n;

	n   = Limit( v , f_min, f_max );	// foCXl E㉺l̃~bg
	max = Limit( n , min,   f_max );	// ŏl̃~bgčőlݒ
	vol = Limit( vol, min, max );		// foCXlőEŏl̃~bgăZbg
}

/** foCX ŏl ݒ
 * @fn     void IO_ctrl::set_min(int v)
 * @param v  : ŏl
 *
 */
void IO_ctrl::set_min(int v)
{
	int n;

	n   = Limit( v , f_min, f_max );	// foCXl E㉺l̃~bg
	min = Limit( n , f_min, max );		// ől̃~bgčŏlݒ
	vol = Limit( vol, min, max );		// foCXlőEŏl̃~bgăZbg
}

/** foCXl ݒ
 * @fn     int IO_ctrl::set_vol(int v)
 * @param v  : foCXl
 * @retval   : ݒ肵foCXl
 *
 */
int IO_ctrl::set_vol(int v)
{
	vol = Limit( v , min, max );		// foCXlőEŏl̃~bgăZbg
	return vol;
}

/** foCXl̉Z
 * @fn     int  IO_ctrl::add(int v)
 * @param v  : Zl
 * @retval   : vZ̃foCXl
 *
 */
int  IO_ctrl::add(int v)
{
	int n;

	n = vol + v;
	vol = Limit( n, min, max );		// ZlőEŏl̃~bgăZbg
	return vol;
}

/** foCXl̉ZiTCNbNE[hj
 * @fn     int  IO_ctrl::addc(int v)
 * @param v  : Zl
 * @retval   : vZ̃foCXl
 *
 */
int  IO_ctrl::addc(int v)
{
	int n;

	n = vol + v;
	vol = Cyclic( n, min, max );		// ZlőEŏl̃TCNbNăZbg
	return vol;
}

/** foCX̏Ԃ擾
 * @fn     void IO_ctrl::view( VIEW_t* v)
 * @param v  : ԏ̊i[|C^
 *
 */
void IO_ctrl::view( VIEW_t* v)
{
	v->dev_if = dev_if;
	v->f_max  = f_max;
	v->f_min  = f_min;
	v->mask   = mask;
	v->max    = max;
	v->min    = min;
	v->mode   = mode;
	v->vol    = vol;
}


//  =========================================
//
//		Axis Input Device(AXIS)@p֐
//
//  =========================================

/** Axis Input Device ̃NX
 * @fn     AXIS_ctrl::AXIS_ctrl(void)
 *
 */
AXIS_ctrl::AXIS_ctrl(void)
{
	f_max = 255;
	f_min = -255;
	max   = 255;
	min   = 0;
	vol   = 127;
	mode  = 0;
	dzone = 0;
}

/** Axis Input Device ̓͏
 * @fn     int AXIS_ctrl::inp(void)
 * @retval : ͒l
 *
 */
int AXIS_ctrl::inp(void)
{
	int axis;
	int v_ax;

	axis = ps2.axis[dev_if.ch_no];
	v_ax = rang_chg_dzone( axis, -32768, 32767, min, max, dzone);
	set_vol( v_ax );
	v_ax = get_vol();
	if (v_ax != 0)
	{
		if ((mode == 0)||(mode == 1)) mode++;
		if ( mode == 3 ) mode = 0;
	}
	else {
		if ((mode > 0)&&(mode < 3)) mode = 3;
		else if (mode == 3) mode = 0;
	}
	return (v_ax);
}


//  =========================================
//
//		Button Input Device(BUTT)
//
//  =========================================

/** Button Input Device ̃NX
 * @fn     BUTT_ctrl::BUTT_ctrl(void)
 *
 */
BUTT_ctrl::BUTT_ctrl(void)
{
	f_max = 1;
	f_min = 0;
	max   = 1;
	min   = 0;
	vol   = 0;
	mode  = 0;
	dzone = 0;
}

/** Button Input Device ̓͏
 * @fn     int BUTT_ctrl::inp(void)
 * @retval : ͒l
 *
 */
int BUTT_ctrl::inp(void)
{
	int btn;
	int v_btn;

	btn   = ps2.BTN_bit & mask ;
	v_btn = 0;
	if (btn) v_btn = 1;
	set_vol( v_btn );
	if (v_btn != 0)
	{
		if ((mode == 0)||(mode == 1)) mode++;
		if ( mode == 3 ) mode = 0;
	}
	else {
		if ((mode > 0)&&(mode < 3)) mode = 3;
		else if (mode == 3) mode = 0;
	}
	return (v_btn);
}

//  =========================================
//
//		AXIS & POV Input Device(POV)
//
//  =========================================

/** AXIS & POV Input Device ̃NX
 * @fn     POV_ctrl::POV_ctrl(void)
 *
 */
POV_ctrl::POV_ctrl(void)
{
	f_max = 1;
	f_min = 0;
	max   = 1;
	min   = 0;
	vol   = 0;
	mode  = 0;
	dzone = 0;
}

/** AXIS & POV Input Device ̓͏
 * @fn     int POV_ctrl::inp(void)
 * @retval : ͒l
 *
 */
int POV_ctrl::inp(void)
{
	int pov;
	int v_pov;

	pov   = ps2.POV_bit & mask ;
	v_pov = 0;
	if (pov) v_pov = 1;
	set_vol( v_pov );
	if (v_pov != 0)
	{
		if ((mode == 0)||(mode == 1)) mode++;
		if ( mode == 3 ) mode = 0;
	}
	else {
		if ((mode > 0)&&(mode < 3)) mode = 3;
		else if (mode == 3) mode = 0;
	}
	return (v_pov);
}


//  =========================================
//
//		DC motor Device(DCMT)
//
//  =========================================

/** DC motor Device ̃NX
 * @fn     DCMT_ctrl::DCMT_ctrl(void)
 *
 */
DCMT_ctrl::DCMT_ctrl(void)
{
	f_max = 255;
	f_min = 0;
	max   = 255;
	min   = 0;
	vol   = 127;
	mode  = 0;
	dzone = 0;
}

/** DC motor Device ̏o͏
 * @fn     void DCMT_ctrl::out(void)
 *
 */
void DCMT_ctrl::out(void)
{
	MT = vol;
}

//  =========================================
//
//		PWM Output Device(PWM)
//
//  =========================================

/** PWM Output Device ̃NX
 * @fn     PWM_ctrl::PWM_ctrl(void)
 *
 */
PWM_ctrl::PWM_ctrl(void)
{
	f_max = 255;
	f_min = 0;
	max   = 255;
	min   = 0;
	vol   = 127;
	mode  = 0;
	dzone = 0;
}

/** PWM2 Output Device ̃NX
 * @fn     PWM_ctrl::PWM_ctrl(void)
 *
 */
PWM2_ctrl::PWM2_ctrl(void)
{
	f_max = 255;
	f_min = 0;
	max   = 255;
	min   = 0;
	vol   = 127;
	mode  = 0;
	dzone = 0;
}

/** DC motor Device ̏o͏
 * @fn     void PWM_ctrl::out(void)
 *
 */
void PWM_ctrl::out(void)
{
	PWM[dev_if.ch_no] = vol;
}

/** PWM ̏o͏
 * @fn     void PWM2_ctrl::out(void)
 *
 */
void PWM2_ctrl::out(void)
{
	PWM2[dev_if.ch_no] = vol;
}


//  =========================================
//
//		Digital Output Device(DO)
//
//  =========================================

/** Digital Output Device ̃NX
 * @fn     PWM_ctrl::PWM_ctrl(void)
 *
 */
DO_ctrl::DO_ctrl(void)
{
	f_max = 1;
	f_min = 0;
	max   = 1;
	min   = 0;
	vol   = 0;
	mode  = 0;
	dzone = 0;
}

/** Digital Output Device ̏o͏
 * @fn     void DO_ctrl::out(void)
 *
 */
void DO_ctrl::out(void)
{
	if (vol) d_out = (d_out & (~mask)) | mask ;
	else d_out = d_out & (~mask);
}


//  =========================================
//
//		Video select port(VSEL)
//
//  =========================================

/** Video select port ̃NX
 * @fn     VSEL_ctrl::VSEL_ctrl(void)
 *
 */
VSEL_ctrl::VSEL_ctrl(void)
{
	f_max = 3;
	f_min = 0;
	max   = 3;
	min   = 0;
	vol   = 0;
	mode  = 0;
	dzone = 0;
}

/** Video select port ̏o͏
 * @fn     void VSEL_ctrl::out(void)
 *
 */
void VSEL_ctrl::out(void)
{
	camera_no = vol;
	Camera_Chg(vol);
}

//  =========================================
//
//		VGA/QVGA select port(VGA)
//
//  =========================================

/** VGA/QVGA select port ̃NX
 * @fn     VGA_ctrl::VGA_ctrl(void)
 *
 */
VGA_ctrl::VGA_ctrl(void)
{
	f_max = 1;
	f_min = 0;
	max   = 1;
	min   = 0;
	vol   = 0;
	mode  = 0;
	dzone = 0;
}

/** VGA/QVGA select port ̏o͏
 * @fn     void VGA_ctrl::out(void)
 *
 */
void VGA_ctrl::out(void)
{
	vga = vol;
	TPJT_set_video_inf(vol);
	TPJT_set_vspeed(JPEG_speed[vga]);
}

//  =========================================
//
//		Action control mode (MODE)
//
//  =========================================

/** VAction control mode ̃NX
 * @fn     MODE_ctrl::MODE_ctrl(void)
 *
 */
MODE_ctrl::MODE_ctrl(void)
{
	f_max = 3;
	f_min = 0;
	max   = 3;
	min   = 0;
	vol   = 0;
	mode  = 0;
	dzone = 0;
}

/** VAction control mode ̏o͏
 * @fn     void MODE_ctrl::out(void)
 *
 */
void MODE_ctrl::out(void)
{
	act_mode = vol;
}

//  =========================================
//
//		Xchange control 
//
//  =========================================

/** Xchange control ̃NX
 * @fn     XCHG_ctrl::XCHG_ctrl(void)
 *
 */
XCHG_ctrl::XCHG_ctrl(void)
{
	in_dev  = 0;
	out_dev = 0;
	coeff   = 0;
	init_vol= 0;
	type    = 0;
	fade	= 0;
}

/** Xchange control ̃NX
 * @fn     void XCHG_ctrl::init(IO_ctrl* in_dev_p,IO_ctrl* out_dev_p, int cont, int init_v, int a_type )
 * @param  in_dev_p  : ̓foCX`NXE|C^
 * @param  out_dev_p : o̓foCX`NXE|C^
 * @param  cont      :
 * @param  init_v    : l
 * @param  a_type    : device type
 *
 */
void XCHG_ctrl::init(IO_ctrl* in_dev_p,IO_ctrl* out_dev_p, int cont, int init_v, int a_type )
{
	in_dev	= in_dev_p;
	out_dev	= out_dev_p;
	coeff	= cont;
	init_vol= init_v;
	type	= a_type;
}
